@@ -12,31 +12,11 @@ else if (file_exists('./plugins/compatibility/functions.php'))
 
 function overrideSmConfig() {
 
-  global $data_dir, 
-         $org_name, $org_logo, $org_logo_width, $org_logo_height,
-         $org_title, $signout_page, $frame_top, $squirrelmail_default_language;
-
-  global $domain, $imapServerAddress, $imapPort, $useSendmail, $smtpServerAddress,
-         $smtpPort, $sendmail_path, $use_authenticated_smtp,
-         $pop_before_smtp, $imap_server_type, $invert_time, 
-         $optional_delimiter;
-         
-  global $default_folder_prefix, $trash_folder, $sent_folder, $draft_folder,
-         $default_move_to_trash, $default_move_to_sent, $default_save_as_draft,
-         $show_prefix_option, $list_special_folders_first, $use_special_folder_color,
-         $auto_expunge, $default_sub_of_inbox, $show_contain_subfolders_option,
-         $default_unseen_notify, $default_unseen_type, $auto_create_special,
-         $delete_folder, $noselect_fix_enable, $plugins, $squirrelmail_plugin_hooks;
-          
-      
-  global $notPartOfDomainName, $numberOfDotSections, 
-         $putHostNameOnFrontOfUsername, $checkByExcludeList, 
-         $at, $dot, $dontUseHostName, $perUserSettingsFile,
-         $smHostIsDomainThatUserLoggedInWith, $virtualDomains, 
-         $sendmailVirtualUserTable, $virtualDomainDataDir, 
-         $allVirtualDomainsAreUnderOneSSLHost, $debug, $removeFromFront, 
-         $chopOffDotSectionsFromRight, $chopOffDotSectionsFromLeft,
-         $translateHostnameTable, $pathToQmail;
+   global $allVirtualDomainsAreUnderOneSSLHost, 
+          $smHostIsDomainThatUserLoggedInWith, $virtualDomainDataDir, 
+          $data_dir, $domain, $virtualDomains, $plugins,  
+          $squirrelmail_plugin_hooks, $useSessionBased, 
+          $lockoutEnabled, $lockoutTable;
       
   
   // get global variable for versions of PHP < 4.1 
@@ -55,14 +35,25 @@ function overrideSmConfig() {
 
 
   if (compatibility_check_sm_version(1, 3))
-     include_once (SM_PATH . 'plugins/vlogin/config.php');
+     include_once (SM_PATH . 'plugins/vlogin/data/config.php');
   else
-     include_once ('../plugins/vlogin/config.php');
+     include_once ('../plugins/vlogin/data/config.php');
+
+
+  if ($useSessionBased)
+  {
+     global $config_override;
+     compatibility_sqsession_unregister('config_override');
+     // If we don't initialize this, it will work 
+     // together with the multilogin plugin
+     // $config_override = array();
+  }
 
 
   // grab hostname into local var
   //
   $hostname = $_SERVER['HTTP_HOST'];
+  $hostname_stripped = deconstructDomainName($hostname);
 
 
   // for sites where virtual host is pegged on the end of the main
@@ -81,20 +72,32 @@ function overrideSmConfig() {
   // flag is on...
   //
   if ($smHostIsDomainThatUserLoggedInWith)
-     $domain = deconstructDomainName($_SERVER['HTTP_HOST']);
+  {
+     if ($useSessionBased)
+        $config_override['domain'] = $hostname_stripped;
+     else
+        $domain = $hostname_stripped; 
+  }
       
 
   // override data_dir if the $virtualDomainDataDir variable 
   // has been specified
   //
   if (!empty($virtualDomainDataDir)) {
-     $data_dir = $virtualDomainDataDir;
-     $data_dir = str_replace('###VIRTUAL_DOMAIN###', 
-                             deconstructDomainName($_SERVER['HTTP_HOST']), 
-                             $data_dir);
+     $the_data_dir = $virtualDomainDataDir;
+     $the_data_dir = str_replace('###VIRTUAL_DOMAIN###', 
+                             $hostname_stripped, 
+                             $the_data_dir);
+     if ($useSessionBased)
+        $config_override['data_dir'] = $the_data_dir;
+     else
+        $data_dir = $the_data_dir;
   }
 
 
+  $firstTime = 1;
+
+
   // override the org_logo and other stuff if we find a match
   //
 // NOTE: the following line will solve PHP 4.3 problems when using
@@ -104,20 +107,67 @@ function overrideSmConfig() {
   if (is_array($virtualDomains))
   foreach (array_keys($virtualDomains) as $virtualDomain) {
 
-    if (stristr($hostname, $virtualDomain)) {
+    if (stristr($hostname, $virtualDomain) || $virtualDomain == '*') {
+
+
+      // limit usage of global domain to first entry only
+      //
+      if ($virtualDomain == '*' && !$firstTime)
+      {
+         echo '<html><body><font color="red" size="12pt">';
+         echo 'Sorry, please contact your administrator and ';
+         echo 'ask them to reconfigure the SquirrelMail Virtual Host Login ';
+         echo 'plugin such that the global virtual domain is listed first';
+         echo '</font></body></html>';
+         exit(1);
+      }
 
 
       foreach ($virtualDomains[$virtualDomain] as $setting => $value)
       {
 
+        // Look for locked out domains, redirect immediately if found
+        //
+        if ($lockoutEnabled)
+        { 
+           if (array_key_exists('###LOCKED OUT###', $virtualDomains[$virtualDomain])
+              && ($virtualDomains[$virtualDomain]['###LOCKED OUT###']))
+           {
+              sqsession_destroy();
+              header('Location: ../plugins/vlogin/' 
+                 . $virtualDomains[$virtualDomain]['###LOCKED OUT###']);
+              exit;
+           }
+           else if ($lockoutTable)
+           {
+              check_lockouts($lockoutTable, '', $hostname);
+           }
+        }
+
+
+        // Go ahead and replace the string ###VIRTUAL_DOMAIN###
+        //
+        if (!is_array($value))
+           if (strpos($value, '###VIRTUAL_DOMAIN###') !== FALSE)
+              $value = str_replace('###VIRTUAL_DOMAIN###',
+                                   $hostname_stripped,
+                                   $value);
+         
         // enable additional plugins
         //
         if (stristr($setting, 'enable_plugins'))
         {
-           foreach ($value as $pluginName)
+           if ($useSessionBased)
            {
-              $plugins[] = $pluginName;
-              use_plugin($pluginName);
+              $config_override[$setting] = $value;
+           }
+           else
+           {
+              foreach ($value as $pluginName)
+              {
+                 $plugins[] = $pluginName;
+                 use_plugin($pluginName);
+              }
            }
            continue;
         }
@@ -127,15 +177,22 @@ function overrideSmConfig() {
         //
         if (stristr($setting, 'disable_plugins'))
         {
-           foreach ($value as $pluginName)
+           if ($useSessionBased)
            {
-              $pluginKey = array_search($pluginName, $plugins);
-              if (!is_null($pluginKey) && $pluginKey !== FALSE)
+              $config_override[$setting] = $value;
+           }
+           else
+           {
+              foreach ($value as $pluginName)
               {
-                 unset($plugins[$pluginKey]);
-                 foreach (array_keys($squirrelmail_plugin_hooks) as $hookName)
+                 $pluginKey = array_search($pluginName, $plugins);
+                 if (!is_null($pluginKey) && $pluginKey !== FALSE)
                  {
-                    unset($squirrelmail_plugin_hooks[$hookName][$pluginName]);
+                    unset($plugins[$pluginKey]);
+                    foreach (array_keys($squirrelmail_plugin_hooks) as $hookName)
+                    {
+                       unset($squirrelmail_plugin_hooks[$hookName][$pluginName]);
+                    }
                  }
               }
            }
@@ -145,29 +202,88 @@ function overrideSmConfig() {
 
         // replace SquirrelMail config values
         //
-        $$setting = $value;
+        if ($useSessionBased)
+        {
+
+           // in order to set the org_title, in the browser
+           // title bar, have to do it now (using an eval
+           // otherwise things like $_SESSION will never be
+           // found when the variable is first defined)
+           //
+           if ($setting == 'org_title')
+           {
+              global $$setting;
+              eval('$$setting = ' . $value . ';');
+           }
+           else
+              $config_override[$setting] = $value;
+
+        }
+        else
+        {
+
+           // in order to set the org_title, in the browser
+           // title bar, have to do it now using and eval,
+           // otherwise things like $_SESSION will never be
+           // found when the variable is first defined)
+           //
+           if ($setting == 'org_title')
+           {
+              global $$setting;
+              eval('$$setting = ' . $value . ';');
+           }
+           else
+           {
+              global $$setting;
+              $$setting = $value;
+           }
+
+        }
 
       }
 
 
       if ($smHostIsDomainThatUserLoggedInWith)
-          $domain = deconstructDomainName($_SERVER['HTTP_HOST']);
+      {
+         if ($useSessionBased)
+            $config_override['domain'] = $hostname_stripped;
+         else
+            $domain = $hostname_stripped; 
+      }
 
       
       if (in_array('data_dir', array_keys($virtualDomains[$virtualDomain]))) {
-          $data_dir = $virtualDomains[$virtualDomain]['data_dir'];
-          $data_dir = str_replace('###VIRTUAL_DOMAIN###',
-                                  deconstructDomainName($_SERVER['HTTP_HOST']),
-                                  $data_dir);
+          if ($useSessionBased)
+             $config_override['data_dir'] = $virtualDomains[$virtualDomain]['data_dir'];
+          else
+             $data_dir = $virtualDomains[$virtualDomain]['data_dir'];
       }
 
-
-      // exit this loop
+      // exit this loop, unless this is the global default
       // 
-      break;
+      if( $virtualDomain != '*' ) break;
 
     }
 
+
+    $firstTime = 0;
+
+
+  }
+
+
+  if ($useSessionBased)
+  {
+
+     compatibility_sqsession_register($config_override, 'config_override');
+
+     if (compatibility_check_sm_version(1, 3))
+        include_once (SM_PATH . 'plugins/multilogin/functions.php');
+     else
+        include_once ('../plugins/multilogin/functions.php');
+
+     multilogin_sqoverride_config_do();
+
   }
 
 
@@ -182,7 +298,6 @@ function overrideSmConfig() {
   perUserOverride($user);
 
 
-
 }
 
 
@@ -191,9 +306,19 @@ function overrideSmConfig() {
 function vlogin_domain_do() {
 
 
-  global $plugins, $login_username, $$login_username, $debug, $data_dir, $foundLoginAlias;
-  global $at, $dot, $dontUseHostName, $sendmailVirtualUserTable, $putHostNameOnFrontOfUsername,
-         $allVirtualDomainsAreUnderOneSSLHost;
+  global $plugins, $login_username, $$login_username, $debug, $data_dir, 
+         $foundLoginAlias, $at, $dot, $dontUseHostName, $atConversion,
+         $sendmailVirtualUserTable, $putHostNameOnFrontOfUsername,
+         $allVirtualDomainsAreUnderOneSSLHost, $prefs_dsn, $useSessionBased,
+         $virtualDomains, $lockoutEnabled, $lockoutTable;
+
+
+  // figure out where prefs are stored
+  //
+  if (isset($prefs_dsn) && !empty($prefs_dsn))
+      $prefsInDB = true;
+  else
+      $prefsInDB = false;
 
 
   // get global variable for versions of PHP < 4.1
@@ -222,9 +347,9 @@ function vlogin_domain_do() {
 
 
   if (compatibility_check_sm_version(1, 3))
-     include_once (SM_PATH . 'plugins/vlogin/config.php');
+     include_once (SM_PATH . 'plugins/vlogin/data/config.php');
   else
-     include_once ('../plugins/vlogin/config.php');
+     include_once ('../plugins/vlogin/data/config.php');
 
 
 
@@ -246,18 +371,29 @@ function vlogin_domain_do() {
     // check if login alias was already processed
     if (isset($foundLoginAlias)) {
 
-      //if ($foundLoginAlias) return;
+      if ($foundLoginAlias) return;
 
     }
     else {
 
-//TODO!!  check if this works with db prefs!
-//also, there might be similar code in the password_forget plugin
       // check for login alias here and return if found
+      // (but only if it is in the domain being used
+      // to log in when dontUseHostName is off)
+      //
       $filename = $data_dir.'login_alias.pref';
-      if (file_exists($filename)) {
-        $loginAlias=getPref($data_dir,'login_alias',$user);
-        if (!empty($loginAlias)) return;
+      if ((!$prefsInDB && file_exists($filename)) || $prefsInDB) {
+         $loginAlias=getPref($data_dir,'login_alias',$user);
+
+         if (!empty($loginAlias))
+         {
+            if (!$dontUseHostName)
+            {
+               if (strpos($loginAlias, deconstructDomainName($hostname)) !== FALSE)
+                  return;
+            }
+            else
+               return;
+         }
       }
 
     }
@@ -267,6 +403,13 @@ function vlogin_domain_do() {
 
   // find domain name.  Only truly important bit.
 
+
+  // convert "at" sign...
+  //
+  foreach ($atConversion as $otherAt)
+     $user = str_replace($otherAt, $at, $user);
+
+
   // check and see if they decided to insert the host anyways
   // or we don't want to use the host name...
   //
@@ -297,14 +440,20 @@ function vlogin_domain_do() {
   }
 
 
+  if ($useSessionBased)
+     overrideSmConfig();
+
+
   // override settings on a per-user basis
   //
   perUserOverride($realname);
 
 
   // if password_forget is loaded, use the obfuscated name
-  if (in_array('password_forget',$plugins) && $$login_username != '') $$login_username=$realname;
-  else $login_username=$realname;
+  if (in_array('password_forget',$plugins) && $$login_username != '')
+     $$login_username=$realname;
+  else
+     $login_username=$realname;
 
 
   // when in debug mode, just dump out final login name and quit
@@ -326,35 +475,59 @@ function vlogin_domain_do() {
 
   }
 
+
+  // look for users trying to log into a domain that is locked out
+  //
+  if ($lockoutEnabled)
+  {
+
+     if (strpos($realname, $at) !== FALSE)
+     {
+
+        $usersDomain = substr($realname, strpos($realname, $at) + 1);
+
+        foreach (array_keys($virtualDomains) as $virtualDomain) {
+
+          if (stristr($usersDomain, $virtualDomain)) {
+
+              if ($virtualDomains[$virtualDomain]['###LOCKED OUT###'])
+              {
+                 sqsession_destroy();
+                 header('Location: ../plugins/vlogin/' 
+                    . $virtualDomains[$virtualDomain]['###LOCKED OUT###']);
+                 exit;
+              }
+
+          }
+
+        }
+
+     }
+     else
+        $usersDomain = '';
+
+
+     if ($lockoutTable)
+     {
+        check_lockouts($lockoutTable, $realname, $usersDomain);
+     }
+
+  }
+
+
 }
 
 
 
 function perUserOverride($user) {
 
-  global $data_dir, $squirrelmail_plugin_hooks, 
-         $org_name, $org_logo, $org_logo_width, $org_logo_height,
-         $org_title, $signout_page, $frame_top, $squirrelmail_default_language;
-
-  global $domain, $imapServerAddress, $imapPort, $useSendmail, $smtpServerAddress,
-         $smtpPort, $sendmail_path, $use_authenticated_smtp,
-         $pop_before_smtp, $imap_server_type, $invert_time,
-         $optional_delimiter;
-
-  global $default_folder_prefix, $trash_folder, $sent_folder, $draft_folder,
-         $default_move_to_trash, $default_move_to_sent, $default_save_as_draft,
-         $show_prefix_option, $list_special_folders_first, $use_special_folder_color,
-         $auto_expunge, $default_sub_of_inbox, $show_contain_subfolders_option,
-         $default_unseen_notify, $default_unseen_type, $auto_create_special,
-         $delete_folder, $noselect_fix_enable;
-
-
-  global $numberOfDotSections, $checkByExcludeList, $at, $dot, $dontUseHostName,
-         $smHostIsDomainThatUserLoggedInWith, $virtualDomainDataDir, $perUserSettingsFile,
-         $removeFromFront, $chopOffDotSectionsFromRight, $chopOffDotSectionsFromLeft,
-         $allVirtualDomainsAreUnderOneSSLHost, $sendmailVirtualUserTable;
+  global $at, $atConversion, $perUserSettingsFile, $useSessionBased;
 
 
+  // convert "at" sign...
+  //
+  foreach ($atConversion as $otherAt)
+     $user = str_replace($otherAt, $at, $user);
 
 
   // override per-user settings
@@ -418,6 +591,17 @@ function perUserOverride($user) {
      {
 
 
+        if ($useSessionBased)
+        {
+           global $config_override;
+           compatibility_sqsession_unregister('config_override');
+           // If we don't initialize this, it will work
+           // together with the multilogin plugin and 
+           // any settings already made for the domain
+           // $config_override = array();
+        }
+
+
         // loop through user's settings 
         //
         foreach ($userSettings as $setting)
@@ -427,7 +611,44 @@ function perUserOverride($user) {
               continue;
 
            $matches = explode('=', $setting);
-           $$matches[0] = $matches[1];
+           if ($useSessionBased)
+           {
+              $config_override[$matches[0]] = $matches[1];
+           }
+           else
+           {
+
+              // in order to set the org_title, in the browser
+              // title bar, have to do it now (using an eval
+              // otherwise things like $_SESSION will never be
+              // found when the variable is first defined)
+              //
+              if ($matches[0] == 'org_title')
+              {
+                 global $$matches[0];
+                 eval('$$matches[0] = ' . $matches[1] . ';');
+              }
+              else
+              {
+                 global $$matches[0];
+                 $$matches[0] = $matches[1];
+              }
+           }
+
+        }
+
+
+        if ($useSessionBased)
+        {
+      
+           compatibility_sqsession_register($config_override, 'config_override');
+      
+           if (compatibility_check_sm_version(1, 3))
+              include_once (SM_PATH . 'plugins/multilogin/functions.php');
+           else
+              include_once ('../plugins/multilogin/functions.php');
+
+           multilogin_sqoverride_config_do();
 
         }
 
@@ -450,9 +671,9 @@ function deconstructDomainName($hostname) {
 
 
   if (compatibility_check_sm_version(1, 3))
-     include_once (SM_PATH . 'plugins/vlogin/config.php');
+     include_once (SM_PATH . 'plugins/vlogin/data/config.php');
   else
-     include_once ('../plugins/vlogin/config.php');
+     include_once ('../plugins/vlogin/data/config.php');
 
 
 
@@ -500,7 +721,8 @@ function deconstructDomainName($hostname) {
       foreach ($notPartOfDomainName as $dotSection)
       {
 
-         $hostname = preg_replace('/' . $dotSection . '($|\.)/', '', $hostname);
+         $hostname = preg_replace('/(^|\.)' . $dotSection . '($|\.)/', 
+                                  '${1}', $hostname);
 
       }
 
@@ -712,4 +934,74 @@ function getSendmailVirtualUser($user, $sendmailVirtualUserTable)
 }
 
 
+
+// parses a table that lists domains and/or users that are
+// to be locked out and not allowed to log in
+//
+function check_lockouts($lockoutTable, $user, $host)
+{
+
+   if ($LOCKOUTTABLE = @fopen ($lockoutTable, 'r'))
+   {
+
+      while (!feof($LOCKOUTTABLE))
+      {
+
+         $line = fgets($LOCKOUTTABLE, 4096);
+         $line = trim($line);
+
+
+         // skip blank lines and comment lines
+         //
+         if (strpos($line, '#') === 0 || strlen($line) < 3)
+            continue;
+
+
+         // if we were given a hostname, see if this is 
+         // a domain lockout line
+         //
+         if (!empty($host) && preg_match('/^\s*domain:\s*(\S+)\s+(.+)\s*$/', $line, $matches))
+         {
+
+            // check for match with hostname, redirect if found
+            //
+            if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'), 
+                          strtoupper($matches[1])) . '$/', strtoupper($host)))
+            {
+               fclose($LOCKOUTTABLE);
+               sqsession_destroy();
+               header('Location: ../plugins/vlogin/' . $matches[2]);
+               exit;
+            }
+
+         }
+
+
+         // if we were given a username, see if this is 
+         // a user lockout line
+         //
+         if (!empty($user) && preg_match('/^\s*user:\s*(\S+)\s+(.+)\s*$/', $line, $matches))
+         {
+
+            // check for match with username, redirect if found
+            //
+            if (preg_match('/^' . str_replace(array('?', '*'), array('\w{1}', '.*?'), 
+                          strtoupper($matches[1])) . '$/', strtoupper($user)))
+            {
+               fclose($LOCKOUTTABLE);
+               sqsession_destroy();
+               header('Location: ../plugins/vlogin/' . $matches[2]);
+               exit;
+            }
+
+         }
+
+      }
+
+      fclose($LOCKOUTTABLE);
+
+   }
+}
+
+
 ?>
